@@ -11,7 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with the program; if not, see <http://www.gnu.org/licenses/>  
+ * License along with the program; if not, see <http://www.gnu.org/licenses/>
  *
  *
  * Authors:
@@ -31,10 +31,6 @@
 #include <glib/gi18n-lib.h>
 
 #include <gtk/gtk.h>
-#include <camel/camel-provider.h>
-#include <camel/camel-url.h>
-#include <camel/camel-service.h>
-#include <camel/camel-folder.h>
 #include <libedataserver/e-xml-hash-utils.h>
 #include <libedataserverui/e-passwords.h>
 #include <libedataserver/e-account.h>
@@ -50,7 +46,7 @@
 
 #define d(x) x
 
-int e_plugin_lib_enable (EPlugin *ep, int enable);
+gint e_plugin_lib_enable (EPlugin *ep, gint enable);
 
 /* Account Setup */
 GtkWidget *org_gnome_exchange_mapi_account_setup (EPlugin *epl, EConfigHookItemFactoryData *data);
@@ -68,23 +64,22 @@ void exchange_mapi_book_commit (EPlugin *epl, EConfigTarget *target);
 gboolean exchange_mapi_cal_check (EPlugin *epl, EConfigHookPageCheckData *data);
 void exchange_mapi_cal_commit (EPlugin *epl, EConfigTarget *target);
 
-
 static ExchangeMAPIAccountListener *config_listener = NULL;
 
-static void 
+static void
 free_mapi_listener ( void )
 {
 	g_object_unref (config_listener);
 }
 
-int
-e_plugin_lib_enable (EPlugin *ep, int enable)
+gint
+e_plugin_lib_enable (EPlugin *ep, gint enable)
 {
 	g_debug ("Loading Exchange MAPI Plugin \n");
 
 	if (!config_listener) {
 		config_listener = exchange_mapi_account_listener_new ();
-	 	g_atexit ( free_mapi_listener );
+		g_atexit ( free_mapi_listener );
 	}
 
 	return 0;
@@ -93,7 +88,7 @@ e_plugin_lib_enable (EPlugin *ep, int enable)
 ExchangeMAPIAccountListener *
 exchange_mapi_accounts_peek_config_listener ()
 {
-	return config_listener; 
+	return config_listener;
 }
 
 enum {
@@ -103,7 +98,7 @@ enum {
   COLS_MAX
 };
 
-/* Callback for ProcessNetworkProfile. If we have more than one username, 
+/* Callback for ProcessNetworkProfile. If we have more than one username,
  we need to let the user select. */
 static uint32_t
 create_profile_callback (struct SRowSet *rowset, gpointer data)
@@ -150,7 +145,7 @@ create_profile_callback (struct SRowSet *rowset, gpointer data)
 
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),
-						     -1, _("User name"), renderer, 
+						     -1, _("Username"), renderer,
 						     "text", COL_MAPI_ACCOUNT, NULL);
 
 	/* Model for TreeView */
@@ -186,7 +181,7 @@ create_profile_callback (struct SRowSet *rowset, gpointer data)
 	       /* Get the index from the selected value */
 		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
 		gtk_tree_selection_get_selected (selection, NULL, &iter);
-		gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, COL_MAPI_INDEX, 
+		gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, COL_MAPI_INDEX,
 				    &index, -1);
 	} else /* If we return a value > available, we are canceling the login.*/
 	       index = rowset->cRows + 1;
@@ -201,13 +196,13 @@ validate_credentials (GtkWidget *widget, EConfig *config)
 {
 	EMConfigTargetAccount *target_account = (EMConfigTargetAccount *)(config->target);
 	CamelURL *url = NULL;
- 	gchar *key = NULL, *password = NULL;
-	const gchar *domain_name = NULL; 
+	gchar *key = NULL, *password = NULL;
+	const gchar *domain_name = NULL;
 
 	url = camel_url_new (e_account_get_string (target_account->account, E_ACCOUNT_SOURCE_URL), NULL);
 	domain_name = camel_url_get_param (url, "domain");
 
-	/* Silently remove domain part from a user name when user enters it as such.
+	/* Silently remove domain part from a username when user enters it as such.
 	   This change will be visible in the UI on new edit open. */
 	if (url->user && strchr (url->user, '\\')) {
 		gchar *tmp, *at;
@@ -220,6 +215,7 @@ validate_credentials (GtkWidget *widget, EConfig *config)
 
 	if (!url->user || !*url->user || !url->host || !*url->host || !domain_name || !*domain_name) {
 		e_notice (NULL, GTK_MESSAGE_ERROR, "%s", _("Server, username and domain name cannot be empty. Please fill them with correct values."));
+		camel_url_free (url);
 		return;
 	}
 
@@ -239,17 +235,34 @@ validate_credentials (GtkWidget *widget, EConfig *config)
 
 	/*Can there be a account without password ?*/
 	if (password && *password && domain_name && *domain_name && *url->user && *url->host) {
-		char *error_msg = NULL;
+		guint32 cp_flags = (camel_url_get_param (url, "ssl") && g_str_equal (camel_url_get_param (url, "ssl"), "1")) ? CREATE_PROFILE_FLAG_USE_SSL : CREATE_PROFILE_FLAG_NONE;
+		gchar *error_msg = NULL;
 		gboolean status = exchange_mapi_create_profile (url->user, password, domain_name,
-								url->host, &error_msg, 
-								(mapi_profile_callback_t) create_profile_callback,
-								url->user);
+								url->host, cp_flags, &error_msg,
+								(mapi_profile_callback_t) create_profile_callback, url->user);
+		if (status) {
+			/* profile was created, try to connect to the server */
+			ExchangeMapiConnection *conn;
+			gchar *profname;
+
+			status = FALSE;
+			profname = exchange_mapi_util_profile_name (url->user, domain_name, url->host, FALSE);
+
+			conn = exchange_mapi_connection_new (profname, password);
+			if (conn) {
+				status = exchange_mapi_connection_connected (conn);
+				g_object_unref (conn);
+			}
+
+			g_free (profname);
+		}
+
 		if (status) {
 			/* Things are successful */
-			gchar *profname = NULL, *uri = NULL; 
+			gchar *profname = NULL, *uri = NULL;
 
-			profname = exchange_mapi_util_profile_name (url->user, domain_name);
-			camel_url_set_param(url, "profile", profname);
+			profname = exchange_mapi_util_profile_name (url->user, domain_name, url->host, FALSE);
+			camel_url_set_param (url, "profile", profname);
 			g_free (profname);
 
 			uri = camel_url_to_string(url, 0);
@@ -259,7 +272,7 @@ validate_credentials (GtkWidget *widget, EConfig *config)
 
 			e_notice (NULL, GTK_MESSAGE_INFO, "%s", _("Authentication finished successfully."));
 		} else {
-			char *e;
+			gchar *e;
 
 			e_passwords_forget_password (EXCHANGE_MAPI_PASSWORD_COMPONENT, key);
 
@@ -286,8 +299,8 @@ domain_entry_changed(GtkWidget *entry, EConfig *config)
 {
 	EMConfigTargetAccount *target = (EMConfigTargetAccount *)(config->target);
 	CamelURL *url = NULL;
-	const char *domain = NULL;
-	char *url_string = NULL;
+	const gchar *domain = NULL;
+	gchar *url_string = NULL;
 
 	url = camel_url_new (e_account_get_string(target->account, E_ACCOUNT_SOURCE_URL), NULL);
 	domain = gtk_entry_get_text (GTK_ENTRY(entry));
@@ -305,26 +318,49 @@ domain_entry_changed(GtkWidget *entry, EConfig *config)
 	camel_url_free (url);
 }
 
+static void
+secure_check_toggled (GtkWidget *check, EConfig *config)
+{
+	EMConfigTargetAccount *target = (EMConfigTargetAccount *)(config->target);
+	CamelURL *url = NULL;
+	gchar *url_string = NULL;
+
+	url = camel_url_new (e_account_get_string (target->account, E_ACCOUNT_SOURCE_URL), NULL);
+
+	camel_url_set_param (url, "ssl", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check)) ? "1" : NULL);
+
+	url_string = camel_url_to_string (url, 0);
+	e_account_set_string (target->account, E_ACCOUNT_SOURCE_URL, url_string);
+	e_account_set_string (target->account, E_ACCOUNT_TRANSPORT_URL, url_string);
+	g_free (url_string);
+
+	camel_url_free (url);
+}
+
 GtkWidget *
 org_gnome_exchange_mapi_account_setup (EPlugin *epl, EConfigHookItemFactoryData *data)
 {
 	EMConfigTargetAccount *target_account;
 	CamelURL *url;
 	GtkWidget *hbox = NULL;
+	gint row;
 
 	target_account = (EMConfigTargetAccount *)data->config->target;
 	url = camel_url_new(e_account_get_string(target_account->account, E_ACCOUNT_SOURCE_URL), NULL);
 
 	/* is NULL on New Account creation */
 	if (url == NULL)
-		return NULL; 
+		return NULL;
 
 	if (!g_ascii_strcasecmp (url->protocol, "mapi")) {
 		GtkWidget *label;
 		GtkWidget *domain_name;
 		GtkWidget *auth_button;
-		const char *domain_value = camel_url_get_param (url, "domain");
-		int row = ((GtkTable *)data->parent)->nrows;
+		GtkWidget *secure_conn;
+		const gchar *domain_value = camel_url_get_param (url, "domain");
+		const gchar *use_ssl = camel_url_get_param (url, "ssl");
+
+		g_object_get (data->parent, "n-rows", &row, NULL);
 
 		/* Domain name & Authenticate Button */
 		hbox = gtk_hbox_new (FALSE, 6);
@@ -344,7 +380,15 @@ org_gnome_exchange_mapi_account_setup (EPlugin *epl, EConfigHookItemFactoryData
 
 		gtk_table_attach (GTK_TABLE (data->parent), label, 0, 1, row, row+1, 0, 0, 0, 0);
 		gtk_widget_show_all (GTK_WIDGET (hbox));
-		gtk_table_attach (GTK_TABLE (data->parent), GTK_WIDGET (hbox), 1, 2, row, row+1, GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0); 
+		gtk_table_attach (GTK_TABLE (data->parent), GTK_WIDGET (hbox), 1, 2, row, row+1, GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
+
+		row++;
+
+		secure_conn = gtk_check_button_new_with_mnemonic (_("_Use secure connection"));
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (secure_conn), use_ssl && g_str_equal (use_ssl, "1"));
+		g_signal_connect (secure_conn, "toggled", G_CALLBACK (secure_check_toggled), data->config);
+		gtk_widget_show (secure_conn);
+		gtk_table_attach (GTK_TABLE (data->parent), GTK_WIDGET (secure_conn), 1, 2, row, row + 1, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
 	}
 
 	camel_url_free (url);
@@ -358,19 +402,19 @@ org_gnome_exchange_mapi_check_options(EPlugin *epl, EConfigHookPageCheckData *da
 	gboolean status = TRUE;
 
 	if (data->pageid != NULL && g_ascii_strcasecmp (data->pageid, "10.receive") == 0) {
-		CamelURL *url = camel_url_new (e_account_get_string(target->account,  
+		CamelURL *url = camel_url_new (e_account_get_string(target->account,
 								    E_ACCOUNT_SOURCE_URL), NULL);
 
 		if (url && url->protocol && g_ascii_strcasecmp (url->protocol, "mapi") == 0) {
 			const gchar *prof = NULL;
 
 			/* We assume that if the profile is set, then the setting is valid. */
- 			prof = camel_url_get_param (url, "profile");
+			prof = camel_url_get_param (url, "profile");
 
 			/*Profile not set. Do not proceed with account creation.*/
 			if (!(prof && *prof))
-			        status = FALSE;
-		} 
+				status = FALSE;
+		}
 
 		if (url)
 			camel_url_free(url);
@@ -386,7 +430,6 @@ enum {
 	NUM_COLS
 };
 
-
 static gboolean
 check_node (GtkTreeStore *ts, ExchangeMAPIFolder *folder, GtkTreeIter iter)
 {
@@ -394,7 +437,7 @@ check_node (GtkTreeStore *ts, ExchangeMAPIFolder *folder, GtkTreeIter iter)
 	mapi_id_t fid;
 
 	ts_model = GTK_TREE_MODEL (ts);
-	
+
 	gtk_tree_model_get (ts_model, &iter, 1, &fid, -1);
 	if (fid && folder->parent_folder_id == fid) {
 		/* Do something */
@@ -425,11 +468,11 @@ add_to_store (GtkTreeStore *ts, ExchangeMAPIFolder *folder)
 	GtkTreeIter iter;
 
 	ts_model = GTK_TREE_MODEL (ts);
-	
+
 	gtk_tree_model_get_iter_first (ts_model, &iter);
 	if (!check_node (ts, folder, iter)) {
 		GtkTreeIter node;
-		gtk_tree_store_append (ts, &node, &iter);		
+		gtk_tree_store_append (ts, &node, &iter);
 		gtk_tree_store_set (ts, &node, NAME_COL, folder->folder_name, FID_COL, folder->folder_id, FOLDER_COL, folder, -1);
 	}
 }
@@ -476,8 +519,8 @@ add_folders (GSList *folders, GtkTreeStore *ts, ExchangeMAPIFolderType folder_ty
 {
 	GSList *tmp = folders;
 	GtkTreeIter iter;
-	char *node = _("Personal Folders");
-	
+	gchar *node = _("Personal Folders");
+
 	/* add all... */
 	gtk_tree_store_append (ts, &iter, NULL);
 	gtk_tree_store_set (ts, &iter, NAME_COL, node, -1);
@@ -553,13 +596,13 @@ exchange_mapi_cursor_change (GtkTreeView *treeview, ESource *source)
 	GtkTreeIter       iter;
 	mapi_id_t pfid;
 	gchar *sfid=NULL;
-	
+
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
 	gtk_tree_selection_get_selected(selection, &model, &iter);
 
 	gtk_tree_model_get (model, &iter, FID_COL, &pfid, -1);
 	sfid = exchange_mapi_util_mapi_id_to_string (pfid);
-	e_source_set_property (source, "parent-fid", sfid); 
+	e_source_set_property (source, "parent-fid", sfid);
 	g_free (sfid);
 }
 
@@ -567,12 +610,14 @@ static GtkWidget *
 exchange_mapi_create (GtkWidget *parent, ESource *source, ExchangeMAPIFolderType folder_type)
 {
 	GtkWidget *vbox, *label, *scroll, *tv;
-	char *uri_text;
+	gchar *uri_text;
+	gint row;
 	GtkCellRenderer *rcell;
 	GtkTreeStore *ts;
 	GtkTreeViewColumn *tvc;
-	const char *acc;
+	const gchar *acc;
 	GSList *folders;
+	ExchangeMapiConnection *conn;
 	mapi_id_t fid = 0;
 
 	uri_text = e_source_get_uri (source);
@@ -580,18 +625,24 @@ exchange_mapi_create (GtkWidget *parent, ESource *source, ExchangeMAPIFolderType
 		return NULL;
 	}
 
-	folders = exchange_mapi_account_listener_peek_folder_list ();
+	folders = NULL;
+	conn = exchange_mapi_connection_find (e_source_get_property (source, "profile"));
+	if (conn && exchange_mapi_connection_connected (conn))
+		folders = exchange_mapi_connection_peek_folders_list (conn);
+
 	acc = e_source_group_peek_name (e_source_peek_group (source));
 	ts = gtk_tree_store_new (NUM_COLS, G_TYPE_STRING, G_TYPE_INT64, G_TYPE_POINTER);
 
 	add_folders (folders, ts, folder_type);
-	
+
+	if (conn)
+		g_object_unref (conn);
+
 	vbox = gtk_vbox_new (FALSE, 6);
 
 	if (folder_type == MAPI_FOLDER_TYPE_CONTACT) {
 		gtk_container_add (GTK_CONTAINER (parent), vbox);
 	} else {
-		gint row;
 		g_object_get (parent, "n-rows", &row, NULL);
 		gtk_table_attach (GTK_TABLE (parent), vbox, 0, 2, row+1, row+2, GTK_FILL|GTK_EXPAND, 0, 0, 0);
 	}
@@ -600,14 +651,14 @@ exchange_mapi_create (GtkWidget *parent, ESource *source, ExchangeMAPIFolderType
 	gtk_widget_show (label);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
-	
+
 	rcell = gtk_cell_renderer_text_new ();
 	tvc = gtk_tree_view_column_new_with_attributes (acc, rcell, "text", NAME_COL, NULL);
 	tv = gtk_tree_view_new_with_model (GTK_TREE_MODEL (ts));
 	gtk_tree_view_append_column (GTK_TREE_VIEW (tv), tvc);
 	g_object_set (tv,"expander-column", tvc, "headers-visible", TRUE, NULL);
 	gtk_tree_view_expand_all (GTK_TREE_VIEW (tv));
-	
+
 	if (e_source_get_property (source, "folder-id")) {
 		exchange_mapi_util_mapi_id_from_string (e_source_get_property (source, "folder-id"), &fid);
 		select_folder (GTK_TREE_MODEL (ts), fid, tv);
@@ -664,7 +715,7 @@ exchange_mapi_book_check (EPlugin *epl, EConfigHookPageCheckData *data)
 {
 	EABConfigTargetSource *t = (EABConfigTargetSource *) data->target;
 	ESource *source = t->source;
-	char *uri_text = e_source_get_uri (source);
+	gchar *uri_text = e_source_get_uri (source);
 
 	if (!uri_text)
 		return TRUE;
@@ -687,18 +738,18 @@ exchange_mapi_book_check (EPlugin *epl, EConfigHookPageCheckData *data)
 	return TRUE;
 }
 
-void 
+void
 exchange_mapi_book_commit (EPlugin *epl, EConfigTarget *target)
 {
 	EABConfigTargetSource *t = (EABConfigTargetSource *) target;
 	ESource *source = t->source;
-	char *uri_text;
+	gchar *uri_text;
 	ESourceGroup *grp;
-	
+
 	uri_text = e_source_get_uri (source);
 	if (uri_text && g_ascii_strncasecmp (uri_text, MAPI_URI_PREFIX, MAPI_PREFIX_LENGTH))
 		return;
-	
+
 	//FIXME: Offline handling
 	grp = e_source_peek_group (source);
 	e_source_set_property (source, "auth", "plain/password");
@@ -715,24 +766,23 @@ exchange_mapi_book_commit (EPlugin *epl, EConfigTarget *target)
 	return;
 }
 
-
 /* New calendar/task list/memo list */
 gboolean
 exchange_mapi_cal_check (EPlugin *epl, EConfigHookPageCheckData *data)
 {
 	ECalConfigTargetSource *t = (ECalConfigTargetSource *)(data->target);
 	ESource *source = t->source;
-	char *uri_text = e_source_get_uri (source);
+	gchar *uri_text = e_source_get_uri (source);
 
 	if (!uri_text)
-		return TRUE; 
+		return TRUE;
 
 	/* FIXME: Offline handling */
 
 	/* not a MAPI account */
 	if (g_ascii_strncasecmp (uri_text, MAPI_URI_PREFIX, MAPI_PREFIX_LENGTH)) {
-		g_free (uri_text); 
-		return TRUE; 
+		g_free (uri_text);
+		return TRUE;
 	}
 
 	g_free (uri_text);
@@ -746,32 +796,33 @@ exchange_mapi_cal_check (EPlugin *epl, EConfigHookPageCheckData *data)
 	return TRUE;
 }
 
-void 
+void
 exchange_mapi_cal_commit (EPlugin *epl, EConfigTarget *target)
 {
+	ExchangeMapiConnection *conn;
 	ECalConfigTargetSource *t = (ECalConfigTargetSource *) target;
 	ESourceGroup *group;
 	ESource *source = t->source;
 	gchar *tmp, *sfid;
 	mapi_id_t fid, pfid;
 	uint32_t type;
-	char *uri_text = e_source_get_uri (source);
+	gchar *uri_text = e_source_get_uri (source);
 
 	if (!uri_text || g_ascii_strncasecmp (uri_text, MAPI_URI_PREFIX, MAPI_PREFIX_LENGTH))
 		return;
 	g_free (uri_text);
 
 	switch (t->source_type) {
-		case E_CAL_SOURCE_TYPE_EVENT: 
-			type = olFolderCalendar; 
+		case E_CAL_SOURCE_TYPE_EVENT:
+			type = olFolderCalendar;
 			break;
-		case E_CAL_SOURCE_TYPE_TODO: 
-			type = olFolderTasks; 
+		case E_CAL_SOURCE_TYPE_TODO:
+			type = olFolderTasks;
 			break;
-		case E_CAL_SOURCE_TYPE_JOURNAL: 
-			type = olFolderNotes; 
+		case E_CAL_SOURCE_TYPE_JOURNAL:
+			type = olFolderNotes;
 			break;
-		default: 
+		default:
 			g_warning ("%s: %s: Unknown ExchangeMAPIFolderType\n", G_STRLOC, G_STRFUNC);
 			return;
 	}
@@ -780,7 +831,12 @@ exchange_mapi_cal_commit (EPlugin *epl, EConfigTarget *target)
 
 	exchange_mapi_util_mapi_id_from_string (e_source_get_property (source, "parent-fid"), &pfid);
 
-	fid = exchange_mapi_create_folder (type, pfid, e_source_peek_name (source));
+	/* the profile should be already connected */
+	conn = exchange_mapi_connection_find (e_source_get_property (source, "profile"));
+	g_return_if_fail (conn != NULL);
+
+	fid = exchange_mapi_connection_create_folder (conn, type, pfid, 0, e_source_peek_name (source));
+	g_object_unref (conn);
 
 	sfid = exchange_mapi_util_mapi_id_to_string (fid);
 	tmp = g_strconcat (";", sfid, NULL);
@@ -797,7 +853,7 @@ exchange_mapi_cal_commit (EPlugin *epl, EConfigTarget *target)
 	tmp = e_source_group_get_property (group, "username");
 	e_source_set_property (source, "username", tmp);
 	g_free (tmp);
-	
+
 	tmp = e_source_group_get_property (group, "host");
 	e_source_set_property (source, "host", tmp);
 	g_free (tmp);
